@@ -1,7 +1,11 @@
 <?php
 // functions for managing Shibboleth options through the WordPress administration panel
 
-add_action('admin_menu', 'shibboleth_admin_panels');
+if ( is_multisite() ) {
+	add_action('network_admin_menu', 'shibboleth_network_admin_panels');
+} else {
+	add_action('admin_menu', 'shibboleth_admin_panels');
+}
 
 /**
  * Setup admin menus for Shibboleth options.
@@ -9,16 +13,32 @@ add_action('admin_menu', 'shibboleth_admin_panels');
  * @action: admin_menu
  **/
 function shibboleth_admin_panels() {
-	// global options page
-	if ( function_exists('is_site_admin') ) {
-		$hookname = add_submenu_page('wpmu-admin.php', __('Shibboleth Options', 'shibboleth'), 
-			__('Shibboleth', 'shibboleth'), 8, 'shibboleth-options', 'shibboleth_options_page' );
-	} else {
-		$hookname = add_options_page(__('Shibboleth options', 'shibboleth'), 
-			__('Shibboleth', 'shibboleth'), 8, 'shibboleth-options', 'shibboleth_options_page' );
-	}
+	$hookname = add_options_page(__('Shibboleth options', 'shibboleth'),
+		__('Shibboleth', 'shibboleth'), 'manage_options', 'shibboleth-options', 'shibboleth_options_page' );
 
-	add_contextual_help($hookname, shibboleth_help_text());
+	$screen = WP_Screen::get($hookname);
+	$screen->add_help_tab(array(
+		'title' => 'Shibboleth Help',
+		'id' => 'shibboleth-help',
+		'content' => shibboleth_help_text(),
+	));
+}
+
+/**
+ * Setup multisite admin menus for Shibboleth options.
+ *
+ * @action: network_admin_menu
+ **/
+function shibboleth_network_admin_panels() {
+	$hookname = add_submenu_page('settings.php', __('Shibboleth options', 'shibboleth'),
+		__('Shibboleth', 'shibboleth'), 'manage_network_options', 'shibboleth-options', 'shibboleth_options_page' );
+
+	$screen = WP_Screen::get($hookname);
+	$screen->add_help_tab(array(
+		'title' => 'Shibboleth Help',
+		'id' => 'shibboleth-help',
+		'content' => shibboleth_help_text(),
+	));
 }
 
 
@@ -32,7 +52,7 @@ function shibboleth_help_text() {
 		<li><a href="https://spaces.internet2.edu/display/SHIB2/" target="_blank">' . __('Shibboleth 2 Wiki', 'shibboleth') . '</a></li>
 		<li><a href="http://shibboleth.internet2.edu/lists.html" target="_blank">' . __('Shibboleth Mailing Lists', 'shibboleth') . '</a></li>
 	</ul>';
-	
+
 	return apply_filters( 'shibboleth_help_text_filter', $text );
 
 }
@@ -45,6 +65,8 @@ function shibboleth_help_text() {
  */
 function shibboleth_options_page() {
 	global $wp_roles;
+	$message = null;
+	$type = null;
 
 	if ( isset($_POST['submit']) ) {
 		check_admin_referer('shibboleth_update_options');
@@ -75,10 +97,19 @@ function shibboleth_options_page() {
 		shibboleth_update_option('shibboleth_logout_url', $_POST['logout_url']);
 		shibboleth_update_option('shibboleth_password_change_url', $_POST['password_change_url']);
 		shibboleth_update_option('shibboleth_password_reset_url', $_POST['password_reset_url']);
-		shibboleth_update_option('shibboleth_default_login', (boolean) $_POST['default_login']);
-		shibboleth_update_option('shibboleth_update_users', (boolean) $_POST['update_users']);
-		shibboleth_update_option('shibboleth_update_roles', (boolean) $_POST['update_roles']);
-		
+		shibboleth_update_option('shibboleth_default_login', !empty($_POST['default_login']));
+		shibboleth_update_option('shibboleth_auto_login', !empty($_POST['auto_login']));
+		shibboleth_update_option('shibboleth_update_users', !empty($_POST['update_users']));
+		shibboleth_update_option('shibboleth_update_roles', !empty($_POST['update_roles']));
+
+		$type = 'updated';
+		$message = __( 'Settings saved.', 'shibboleth' );
+
+		if ( function_exists( 'add_settings_error' ) ) {
+			add_settings_error( 'shibboleth_settings_updated', esc_attr( 'shibboleth_settings_updated' ), $message, $type );
+			settings_errors( 'shibboleth_settings_updated' );
+		}
+
 		/**
 		 * action shibboleth_form_submit
 		 * @since 1.4
@@ -106,39 +137,39 @@ function shibboleth_options_page() {
 
 			<table class="form-table">
 				<tr valign="top">
-					<th scope="row"><label for="login_url"><?php _e('Session Initiator URL', 'shibboleth') ?></label</th>
+					<th scope="row"><label for="login_url"><?php _e('Session Initiator URL', 'shibboleth') ?></label></th>
 					<td>
 						<input type="text" id="login_url" name="login_url" value="<?php echo shibboleth_get_option('shibboleth_login_url') ?>" size="50" /><br />
 						<?php _e('This URL is constructed from values found in your main Shibboleth'
 							. ' SP configuration file: your site hostname, the Sessions handlerURL,'
 							. ' and the SessionInitiator Location.', 'shibboleth'); ?>
-						<br /><?php _e('Wiki Documentation', 'shibboleth') ?>: 
+						<br /><?php _e('Wiki Documentation', 'shibboleth') ?>:
 						<a href="https://spaces.internet2.edu/display/SHIB/SessionInitiator" target="_blank">Shibboleth 1.3</a> |
 						<a href="https://spaces.internet2.edu/display/SHIB2/NativeSPSessionInitiator" target="_blank">Shibboleth 2</a>
 					</td>
 				</tr>
 				<tr valign="top">
-					<th scope="row"><label for="logout_url"><?php _e('Logout URL', 'shibboleth') ?></label</th>
+					<th scope="row"><label for="logout_url"><?php _e('Logout URL', 'shibboleth') ?></label></th>
 					<td>
 						<input type="text" id="logout_url" name="logout_url" value="<?php echo shibboleth_get_option('shibboleth_logout_url') ?>" size="50" /><br />
 						<?php _e('This URL is constructed from values found in your main Shibboleth'
 							. ' SP configuration file: your site hostname, the Sessions handlerURL,'
 							. ' and the LogoutInitiator Location (also known as the'
 							. ' SingleLogoutService Location in Shibboleth 1.3).', 'shibboleth'); ?>
-						<br /><?php _e('Wiki Documentation', 'shibboleth') ?>: 
+						<br /><?php _e('Wiki Documentation', 'shibboleth') ?>:
 						<a href="https://spaces.internet2.edu/display/SHIB/SPMainConfig" target="_blank">Shibboleth 1.3</a> |
 						<a href="https://spaces.internet2.edu/display/SHIB2/NativeSPLogoutInitiator" target="_blank">Shibboleth 2</a>
 					</td>
 				</tr>
 				<tr valign="top">
-					<th scope="row"><label for="password_change_url"><?php _e('Password Change URL', 'shibboleth') ?></label</th>
+					<th scope="row"><label for="password_change_url"><?php _e('Password Change URL', 'shibboleth') ?></label></th>
 					<td>
 						<input type="text" id="password_change_url" name="password_change_url" value="<?php echo shibboleth_get_option('shibboleth_password_change_url') ?>" size="50" /><br />
 						<?php _e('If this option is set, Shibboleth users will see a "change password" link on their profile page directing them to this URL.', 'shibboleth') ?>
 					</td>
 				</tr>
 				<tr valign="top">
-					<th scope="row"><label for="password_reset_url"><?php _e('Password Reset URL', 'shibboleth') ?></label</th>
+					<th scope="row"><label for="password_reset_url"><?php _e('Password Reset URL', 'shibboleth') ?></label></th>
 					<td>
 						<input type="text" id="password_reset_url" name="password_reset_url" value="<?php echo shibboleth_get_option('shibboleth_password_reset_url') ?>" size="50" /><br />
 						<?php _e('If this option is set, Shibboleth users who try to reset their forgotten password using WordPress will be redirected to this URL.', 'shibboleth') ?>
@@ -155,12 +186,22 @@ function shibboleth_options_page() {
 							. ' initiated from the WordPress login form by clicking the "Login with Shibboleth" link.', 'shibboleth'); ?></p>
 					</td>
 				</tr>
+				<tr>
+				<th scope="row"><label for="auto_login"><?php _e('Shibboleth automatic login', 'shibboleth') ?></label></th>
+					<td>
+						<input type="checkbox" id="auto_login" name="auto_login" <?php echo shibboleth_get_option('shibboleth_auto_login') ? ' checked="checked"' : '' ?> />
+						<label for="auto_login"><?php _e('Use Shibboleth to auto-login users.', 'shibboleth'); ?></label>
+
+						<p><?php _e('If set, this will force a wp_signon() call and wp_safe_redirect()'
+							. ' to the site_url option.' , 'shibboleth'); ?></p>
+					</td>
+				</tr>
 <?php
 	/**
 	 * action shibboleth_options_table
 	 * Add your own Shibboleth options items to the Shibboleth options table.
 	 * Note: This is in a <table> so add a <tr> with appropriate styling.
-	 * 
+	 *
 	 * @param $shib_headers array
 	 * @param $shib_roles array
 	 * @since 1.4
@@ -178,52 +219,52 @@ function shibboleth_options_page() {
 				. ' <code>AAP.xml</code> (for Shibboleth 1.x).', 'shibboleth') ?></p>
 
 			<p>
-				<?php _e('Wiki Documentation', 'shibboleth') ?>: 
+				<?php _e('Wiki Documentation', 'shibboleth') ?>:
 				<a href="https://spaces.internet2.edu/display/SHIB/AttributeAcceptancePolicy" target="_blank">Shibboleth 1.3</a> |
 				<a href="https://spaces.internet2.edu/display/SHIB2/NativeSPAddAttribute" target="_blank">Shibboleth 2</a>
 			</p>
 
 			<table class="form-table optiontable editform" cellspacing="2" cellpadding="5">
 				<tr valign="top">
-					<th scope="row"><label for="username"><?php _e('Username') ?></label</th>
-					<td><input type="text" id="username" name="headers[username][name]" value="<?php echo 
+					<th scope="row"><label for="username"><?php _e('Username') ?></label></th>
+					<td><input type="text" id="username" name="headers[username][name]" value="<?php echo
 						$shib_headers['username']['name'] ?>" /></td>
 					<td width="60%"></td>
 				</tr>
 				<tr valign="top">
-					<th scope="row"><label for="first_name"><?php _e('First name') ?></label</th>
-					<td><input type="text" id="first_name" name="headers[first_name][name]" value="<?php echo 
+					<th scope="row"><label for="first_name"><?php _e('First name') ?></label></th>
+					<td><input type="text" id="first_name" name="headers[first_name][name]" value="<?php echo
 						$shib_headers['first_name']['name'] ?>" /></td>
-					<td><input type="checkbox" id="first_name_managed" name="headers[first_name][managed]" <?php 
-						checked($shib_headers['first_name']['managed'], 'on') ?> /> <?php _e('Managed', 'shibboleth') ?></td>
+					<td><input type="checkbox" id="first_name_managed" name="headers[first_name][managed]" <?php
+						if (isset($shib_headers['first_name']['managed'])) checked($shib_headers['first_name']['managed'], 'on') ?> /> <?php _e('Managed', 'shibboleth') ?></td>
 				</tr>
 				<tr valign="top">
-					<th scope="row"><label for="last_name"><?php _e('Last name') ?></label</th>
-					<td><input type="text" id="last_name" name="headers[last_name][name]" value="<?php echo 
+					<th scope="row"><label for="last_name"><?php _e('Last name') ?></label></th>
+					<td><input type="text" id="last_name" name="headers[last_name][name]" value="<?php echo
 						$shib_headers['last_name']['name'] ?>" /></td>
-					<td><input type="checkbox" id="last_name_managed" name="headers[last_name][managed]" <?php 
-						checked($shib_headers['last_name']['managed'], 'on') ?> /> <?php _e('Managed', 'shibboleth') ?></td>
+					<td><input type="checkbox" id="last_name_managed" name="headers[last_name][managed]" <?php
+						if (isset($shib_headers['last_name']['managed'])) checked($shib_headers['last_name']['managed'], 'on') ?> /> <?php _e('Managed', 'shibboleth') ?></td>
 				</tr>
 				<tr valign="top">
-					<th scope="row"><label for="nickname"><?php _e('Nickname') ?></label</th>
-					<td><input type="text" id="nickname" name="headers[nickname][name]" value="<?php echo 
+					<th scope="row"><label for="nickname"><?php _e('Nickname') ?></label></th>
+					<td><input type="text" id="nickname" name="headers[nickname][name]" value="<?php echo
 						$shib_headers['nickname']['name'] ?>" /></td>
-					<td><input type="checkbox" id="nickname_managed" name="headers[nickname][managed]" <?php 
-						checked($shib_headers['nickname']['managed'], 'on') ?> /> <?php _e('Managed', 'shibboleth') ?></td>
+					<td><input type="checkbox" id="nickname_managed" name="headers[nickname][managed]" <?php
+						if (isset($shib_headers['nickname']['managed'])) checked($shib_headers['nickname']['managed'], 'on') ?> /> <?php _e('Managed', 'shibboleth') ?></td>
 				</tr>
 				<tr valign="top">
-					<th scope="row"><label for="_display_name"><?php _e('Display name', 'shibboleth') ?></label</th>
-					<td><input type="text" id="_display_name" name="headers[display_name][name]" value="<?php echo 
+					<th scope="row"><label for="_display_name"><?php _e('Display name', 'shibboleth') ?></label></th>
+					<td><input type="text" id="_display_name" name="headers[display_name][name]" value="<?php echo
 						$shib_headers['display_name']['name'] ?>" /></td>
-					<td><input type="checkbox" id="display_name_managed" name="headers[display_name][managed]" <?php 
-						checked($shib_headers['display_name']['managed'], 'on') ?> /> <?php _e('Managed', 'shibboleth') ?></td>
+					<td><input type="checkbox" id="display_name_managed" name="headers[display_name][managed]" <?php
+						if (isset($shib_headers['display_name']['managed'])) checked($shib_headers['display_name']['managed'], 'on') ?> /> <?php _e('Managed', 'shibboleth') ?></td>
 				</tr>
 				<tr valign="top">
-					<th scope="row"><label for="email"><?php _e('Email Address', 'shibboleth') ?></label</th>
-					<td><input type="text" id="email" name="headers[email][name]" value="<?php echo 
+					<th scope="row"><label for="email"><?php _e('Email Address', 'shibboleth') ?></label></th>
+					<td><input type="text" id="email" name="headers[email][name]" value="<?php echo
 						$shib_headers['email']['name'] ?>" /></td>
-					<td><input type="checkbox" id="email_managed" name="headers[email][managed]" <?php 
-						checked($shib_headers['email']['managed'], 'on') ?> /> <?php _e('Managed', 'shibboleth') ?></td>
+					<td><input type="checkbox" id="email_managed" name="headers[email][managed]" <?php
+						if (isset($shib_headers['email']['managed'])) checked($shib_headers['email']['managed'], 'on') ?> /> <?php _e('Managed', 'shibboleth') ?></td>
 				</tr>
 			</table>
 
@@ -240,11 +281,11 @@ function shibboleth_options_page() {
 /**
  * filter shibboleth_role_mapping_override
  * Return true to override the default user role mapping form
- * 
+ *
  * @param boolean - default value false
  * @return boolean - true if override
  * @since 1.4
- * 
+ *
  * Use in conjunction with shibboleth_role_mapping_form action below
  */
 if ( apply_filters('shibboleth_role_mapping_override',false) === false ):
@@ -291,7 +332,7 @@ if ( apply_filters('shibboleth_role_mapping_override',false) === false ):
 					foreach ($wp_roles->role_names as $key => $name) {
 						echo'
 						<tr valign="top">
-							<th scope="row">' . _c($name) . '</th>
+							<th scope="row">' . __($name) . '</th>
 							<td><input type="text" id="role_'.$key.'_header" name="shibboleth_roles['.$key.'][header]" value="' . @$shib_roles[$key]['header'] . '" style="width: 100%" /></td>
 							<td><input type="text" id="role_'.$key.'_value" name="shibboleth_roles['.$key.'][value]" value="' . @$shib_roles[$key]['value'] . '" style="width: 100%" /></td>
 						</tr>';
@@ -311,7 +352,7 @@ if ( apply_filters('shibboleth_role_mapping_override',false) === false ):
 <?php
 			foreach ($wp_roles->role_names as $key => $name) {
 				echo '
-						<option value="' . $key . '"' . ($shib_roles['default'] == $key ? ' selected="selected"' : '') . '>' . _c($name) . '</option>';
+						<option value="' . $key . '"' . ($shib_roles['default'] == $key ? ' selected="selected"' : '') . '>' . __($name) . '</option>';
 			}
 ?>
 						</select>
@@ -337,15 +378,15 @@ if ( apply_filters('shibboleth_role_mapping_override',false) === false ):
 			</table>
 
 <?php
-else: 
+else:
 	/**
 	 * action shibboleth_role_mapping_form
 	 * Roll your own custom Shibboleth role mapping admin UI
-	 * 
+	 *
 	 * @param $shib_headers array
 	 * @param $shib_roles array
 	 * @since 1.4
-	 * 
+	 *
 	 * Use in conjunction with shibboleth_role_mapping_override filter
 	 */
 	do_action( 'shibboleth_role_mapping_form', $shib_headers, $shib_roles );
